"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Db = void 0;
const pg_listen_1 = __importDefault(require("pg-listen"));
class Db {
    constructor(pgClientConfig) {
        this.pgSubscriber = pg_listen_1.default(pgClientConfig, {
            parse: (value) => value
        });
    }
    init({ onDbSetup }) {
        return __awaiter(this, void 0, void 0, function* () {
            this.pgSubscriber.events.on('connected', () => __awaiter(this, void 0, void 0, function* () {
                console.log('DbClient.pgSubscriber: Connected');
                yield onDbSetup();
            }));
            this.pgSubscriber.events.on('reconnect', (attempt) => {
                console.warn(`DbClient.pgSubscriber: Reconnecting attempt ${attempt}`);
            });
            this.pgSubscriber.events.on('error', (error) => {
                console.error('DbClient.pgSubscriber: Fatal database connection error:', error);
                process.exit(1);
            });
            this.pgSubscriber.notifications.on('cardano_db_sync_startup', (payload) => __awaiter(this, void 0, void 0, function* () {
                switch (payload) {
                    case 'init':
                        console.log('DbClient.pgSubscriber: cardano-db-sync-extended starting, schema will be reset');
                        break;
                    case 'db-setup':
                        yield onDbSetup();
                        break;
                    default:
                        console.error(`DbClient.pgSubscriber: Unknown message payload ${payload}`);
                }
            }));
            try {
                yield this.pgSubscriber.connect();
                yield this.pgSubscriber.listenTo('cardano_db_sync_startup');
            }
            catch (error) {
                console.error(error);
            }
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.pgSubscriber.close();
        });
    }
}
exports.Db = Db;
//# sourceMappingURL=Db.js.map