"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HasuraClient = void 0;
const apollo_boost_1 = require("apollo-boost");
const apollo_link_http_1 = require("apollo-link-http");
const util_1 = __importDefault(require("@cardano-graphql/util"));
const child_process_1 = require("child_process");
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const dayjs_1 = __importDefault(require("dayjs"));
const utc_1 = __importDefault(require("dayjs/plugin/utc"));
const graphql_1 = require("graphql");
const wrap_1 = require("@graphql-tools/wrap");
const p_retry_1 = __importDefault(require("p-retry"));
const path_1 = __importDefault(require("path"));
dayjs_1.default.extend(utc_1.default);
class HasuraClient {
    constructor(hasuraCliPath, hasuraUri) {
        this.hasuraCliPath = hasuraCliPath;
        this.hasuraUri = hasuraUri;
        this.client = new apollo_boost_1.ApolloClient({
            cache: new apollo_boost_1.InMemoryCache({
                addTypename: false
            }),
            defaultOptions: {
                query: {
                    fetchPolicy: 'network-only'
                }
            },
            link: apollo_link_http_1.createHttpLink({
                uri: `${this.hasuraUri}/v1/graphql`,
                fetch: cross_fetch_1.default,
                headers: {
                    'X-Hasura-Role': 'cardano-graphql'
                }
            })
        });
    }
    applySchemaAndMetadata() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.applyingSchemaAndMetadata)
                return;
            this.applyingSchemaAndMetadata = true;
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.hasuraCli('migrate apply --down all');
                yield this.hasuraCli('migrate apply --up all');
                yield this.hasuraCli('metadata clear');
                yield this.hasuraCli('metadata apply');
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Applying PostgreSQL schema and Hasura metadata')
            });
            this.applyingSchemaAndMetadata = false;
        });
    }
    buildHasuraSchema() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.applySchemaAndMetadata();
            const executor = ({ document, variables }) => __awaiter(this, void 0, void 0, function* () {
                const query = graphql_1.print(document);
                try {
                    const fetchResult = yield cross_fetch_1.default(`${this.hasuraUri}/v1/graphql`, {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'X-Hasura-Role': 'cardano-graphql'
                        },
                        body: JSON.stringify({ query, variables })
                    });
                    return fetchResult.json();
                }
                catch (error) {
                    console.error(error);
                    throw error;
                }
            });
            const coreTypes = [
                'Block',
                'Cardano',
                'Epoch',
                'Block',
                'Transaction'
            ];
            const schema = wrap_1.wrapSchema({
                schema: yield wrap_1.introspectSchema(executor),
                executor
            });
            for (const t of coreTypes) {
                const gqlType = schema.getType(t);
                if (!gqlType) {
                    throw new Error(`Remote schema is missing ${t}`);
                }
            }
            return schema;
        });
    }
    getMeta() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.client.query({
                query: apollo_boost_1.gql `query {
          cardano {
              tip {
                  forgedAt
              }
          }}`
            });
            const { tip } = (_a = result.data) === null || _a === void 0 ? void 0 : _a.cardano[0];
            const currentUtc = dayjs_1.default().utc();
            const tipUtc = dayjs_1.default.utc(tip.forgedAt);
            return {
                initialized: tipUtc.isAfter(currentUtc.subtract(120, 'second')),
                syncPercentage: (tipUtc.valueOf() / currentUtc.valueOf()) * 100
            };
        });
    }
    hasuraCli(command) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                child_process_1.exec(`${this.hasuraCliPath} --skip-update-check --project ${path_1.default.resolve(__dirname, '..', 'hasura', 'project')} --endpoint ${this.hasuraUri} ${command}`, (error, stdout) => {
                    if (error) {
                        reject(error);
                    }
                    console.log(stdout);
                    resolve();
                });
            });
        });
    }
}
exports.HasuraClient = HasuraClient;
//# sourceMappingURL=HasuraClient.js.map